(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: monitor.ml,v 1.8 2014/05/06 12:56:21 fclement Exp $ *)

module Config = Configuration;;

open Refinement_types;;
open Refinement_types_print;;

let interruption () =
  if Config.get_incremental_mode () then
    let ppf = Config.get_formatter () in
    Format.fprintf ppf "Hit <Enter> to continue...@.";
    ignore (input_line stdin);
;;

type 'a sub_zone_report = zone_number * Upervasives.length * 'a;;
type 'a zone_splitting_report =
  zone_number * 'a sub_zone_report * 'a sub_zone_report
;;

let print_zone_number ppf zn =
  Format.fprintf ppf "zone #%a" print_zone_number zn
;;

let print_sub_zone_report print_scalar ppf (zn, card, par) =
  Format.fprintf ppf "  -> %a, card = %a, par = %a"
    print_zone_number zn
    Upervasives_print.print_length card
    print_scalar par
;;

let print_zone_splitting_report print_scalar ppf (zn, szf, szt) =
  Format.fprintf ppf "\nsplit %a@ %a@ %a@ "
    print_zone_number zn
    (print_sub_zone_report print_scalar) szf
    (print_sub_zone_report print_scalar) szt
;; 

let print_iteration_report print_scalar ppf (i, j) =
  Format.fprintf ppf "@[iter = %a, j = %a@]@."
    Unatural_print.print_t i
    print_scalar j
;;

type memory_unit =
  | B | KiB | MiB | GiB | TiB | PiB | EiB | ZiB | YiB
;;

let string_of_memory_unit = function
  | B -> "bytes"
  | KiB -> "kibibytes"
  | MiB -> "mebibytes"
  | GiB -> "gibibytes"
  | TiB -> "tebibytes"
  | PiB -> "pebibytes"
  | EiB -> "exbibytes"
  | ZiB -> "zebibytes"
  | YiB -> "yobibytes"
;;

let total_allocated_memory () =
  let tam = Gc.allocated_bytes () in
  let rec loop ((x, _) as accu) = function
    | [] -> accu
    | mu :: l -> if x < 1024. then accu else loop (x /. 1024., mu) l in
  let (x, mu) = loop (tam, B) [ KiB; MiB; GiB; TiB; PiB; EiB; ZiB; YiB; ] in
  (x, string_of_memory_unit mu)
;;
