(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: zonation.mli,v 1.21 2014/06/11 15:45:35 fclement Exp $ *)

open Refinement_types;;

val same : 'a zonation -> 'a zonation -> bool;;

val equal_coarse_parameter : 'a partition -> 'a zonation -> bool;;

val last_zone_number : 'a zonation -> zone_number;;

(** {6 Partitions} *)

val partition_unique_domains :
  'a partition -> Unatural.t -> 'a partition -> 'a partition
;;

val partition_of_gradient : 'a partition -> 'b gradient -> 'b partition;;

val partition_to_geometry_and_values :
  'a partition -> Unatural.t array * 'a array
;;
(*val partition_of_geometry_and_values :
  Unatural.t array -> 'a array -> 'a partition
;;*)
val one_zone_partition : Umatrix_nonempty_set.size -> 'a -> 'a partition;;

(** {6 Converters} *)

val to_parameter : 'a zonation -> 'a parameter;;

val of_partition :
  ('a gradient -> zone_number -> 'a sub_parameter ->
    'a candidate_cutting list) ->
  'a partition -> 'a zonation
;;
val to_partition : 'a zonation -> 'a partition;;

(** {6 Iterators} *)

val iter : (zone_number -> 'a zone -> unit) -> 'a zonation -> unit;;

val fold_left :
  ('accu -> zone_number -> 'a zone -> 'accu) ->
  'accu -> 'a zonation -> 'accu
;;

val partition_iter :
  (zone_number -> 'a sub_parameter -> unit) -> 'a partition -> unit
;;

val iter_to_do :
  (zone_number -> 'a sub_parameter -> unit) -> 'a zonation -> unit
;;
val iter_value :
  (zone_number -> 'a sub_parameter -> unit) -> 'a zonation -> unit
;;
val fold_left_value :
  ('accu -> zone_number -> 'a sub_parameter -> 'accu) ->
  'accu -> 'a zonation -> 'accu
;;

val force : 'a zonation -> 'a forced_zonation;;

(** {6 Cuttings} *)

val dichotomies_from_family :
  bounds -> Umatrix_nonempty_set.t -> Unatural.t ->
  basic_cutting_family -> Umatrix_nonempty_set.dichotomy list
;;

(*
val better_cutting :
  'a best_component_cutting -> zone_number -> 'a zone ->
  'a best_component_cutting
;;
*)
